/**
 * Using JCE (Java Cryptography Entension) which is integerated
 * with the SDK since (J2SE) version 1.4.0.
 *
 * This is an example of encryption and decryption of a string
 * using symmetric keys. In this case, the same key is used
 * for both encryption and decryption.
 *
 * The algorithm used to generate the key is a variant of DES
 * (The Data Encryption Standard). DES is a popular symmetric
 * block algorithm for data encryption, but because the the key
 * space is relatively small, brute force attacks are possible.
 *
 * Since the DES algorithm is out of date, DESede, a triple DES
 * variant, can be used which increases the key space and helps
 * prevent brute force attacks.
 *
 * Note: AES (The Advanced Encryption Standard) has now replaced
 * DES.   The National Institute of Standards and Technology (NIST)
 * chose "Rijndael" as AES's implementing algorithm. AES well be
 * seen later when examining another "provider".
 */

import javax.crypto.*;
import java.security.*;
public class IncryptDecrypt
{
   private static SecretKey key = null;         // in java.security
   private static Cipher cipher = null;         // in javax.crypto

   /*
    * Lots of exceptions can be thrown here.  I've chosen
    * not to deal with them and just let the application
    * fail.  These exceptions are:
    *                              javax.crypto.IllegalBlockException
    *                              javax.crypto.BadPaddingException
    *                              java.securty.InvalidKeyException
    */

   public static void main(String[] args) throws Exception
   {
      // This statement is not needed since the SunJCE is
      // (statically) installed as of SDK 1.4.  Do this to
      // dynamically install another provider.

      Security.addProvider(new com.sun.crypto.provider.SunJCE());

      // Generate a secret key for a symmetric algorithm and
      // create a Cipher instance. DESede key size is always
      // 168 bits. Other algorithms, like "blowfish", allow
      // for variable lenght keys.

      KeyGenerator keyGenerator =
         KeyGenerator.getInstance("DESede");
      keyGenerator.init(168);
      SecretKey secretKey = keyGenerator.generateKey();
      cipher = Cipher.getInstance("DESede");

      // Store the string as an array of bytes.  You should
      // specify the encoding method for consistent encoding
      // and decoding across different platforms.

      String clearText = "I sure love working with the JCE.";
      byte[] clearTextBytes = clearText.getBytes("UTF8");

      // Initialize the cipher and encrypt this byte array

      cipher.init(Cipher.ENCRYPT_MODE, secretKey);
      byte[] cipherBytes = cipher.doFinal(clearTextBytes);
      String cipherText = new String(cipherBytes, "UTF8");

      // Reinitialize the cipher an decrypt the byte array

      cipher.init(Cipher.DECRYPT_MODE, secretKey);
      byte[] decryptedBytes = cipher.doFinal(cipherBytes);
      String decryptedText = new String(decryptedBytes, "UTF8");

      System.out.println("Before encryption: " + clearText);
      System.out.println("After encryption: " + cipherText);
      System.out.println("After decryption: " + decryptedText);
   }
}
/*
Before encryption: I sure love working with the JCE.
After encryption: ?b???a?P?^2'???U-Y?k?Xis?M5E?????1f?8?
After decryption: I sure love working with the JCE.
*/